#pragma once

#include <filesystem>

// Base Class
#include "ProviderSegment.h"
#include "detail/CoreSegmentBase.h"
#include "detail/mixins/DiskStorageMixin.h"
#include "detail/mixins/MongoDBStorageMixin.h"

namespace armarx::armem::server::ltm
{
    class CoreSegment :
        public detail::CoreSegmentBase<ProviderSegment>,
        public detail::mixin::DiskMemoryItemMixin,
        public detail::mixin::MongoDBStorageMixin
    {
    public:
        CoreSegment(const detail::mixin::Path&,
                    const detail::mixin::MongoDBSettings&,
                    const std::string&,
                    const MemoryID&,
                    const std::shared_ptr<Processors>&);

        bool forEachProviderSegment(std::function<void(ProviderSegment&)> func) const override;
        bool hasProviderSegment(const std::string& name) const override;
        std::shared_ptr<ProviderSegment> findProviderSegment(const std::string&) const override;

    protected:
        void _loadAllReferences(armem::wm::CoreSegment&) override;
        void _resolve(armem::wm::CoreSegment&) override;
        void _store(const armem::wm::CoreSegment&) override;
        void _storeWithoutFiltering(const armem::wm::CoreSegment&) override;

    private:
    };

} // namespace armarx::armem::server::ltm
