#pragma once

#include <filesystem>

// Base Class
#include "detail/MemoryBase.h"
#include "detail/mixins/BufferedMemoryMixin.h"
#include "detail/mixins/CachedMemoryMixin.h"
#include "detail/mixins/DiskStorageMixin.h"
#include "detail/mixins/MongoDBStorageMixin.h"

// Segmnet Type
#include "CoreSegment.h"

namespace armarx::armem::server::ltm
{
    /// @brief A memory storing data on the hard drive and in mongodb (needs 'armarx memory start' to start the mongod instance)
    class Memory :
        public detail::MemoryBase<CoreSegment>,
        public detail::mixin::BufferedMemoryMixin<CoreSegment>,
        public detail::mixin::CachedMemoryMixin<CoreSegment>,
        public detail::mixin::DiskMemoryItemMixin,
        public detail::mixin::MongoDBStorageMixin
    {
    public:
        using MemoryBase = detail::MemoryBase<CoreSegment>;
        using BufferedBase = detail::mixin::BufferedMemoryMixin<CoreSegment>;
        using CachedBase = detail::mixin::CachedMemoryMixin<CoreSegment>;
        using DiskMemoryBase = detail::mixin::DiskMemoryItemMixin;
        using MongoDBStorageMixin = detail::mixin::MongoDBStorageMixin;

        Memory();
        Memory(const detail::mixin::Path& basePath, const std::string& exportName, const std::string& memoryName);
        Memory(const detail::mixin::Path& basePath,
               const detail::mixin::MongoDBSettings&,
               const std::string& exportName,
               const std::string& memoryName);

        void _setExportName(const std::string& n) final;
        void _setMemoryID(const MemoryID& id) final;
        void _enable() final;
        void _disable() final;
        void _configure(const nlohmann::json& config) final;

        bool forEachCoreSegment(std::function<void(CoreSegment&)> func) const final;
        bool hasCoreSegment(const std::string& name) const final;
        std::unique_ptr<CoreSegment>
        findCoreSegment(const std::string& coreSegmentName) const final;

    protected:
        void _loadAllReferences(armem::wm::Memory&) final;
        void _resolve(armem::wm::Memory&) final;
        void _store(const armem::wm::Memory&) final;
        void _storeWithoutFiltering(const armem::wm::Memory&) final;
        void _directlyStore(const armem::wm::Memory&) final;
        void _directlyStoreWithoutFiltering(const armem::wm::Memory&) final;
    };
} // namespace armarx::armem::server::ltm
