#include "NLohmannJSONConverter.h"

#include <ArmarXCore/core/logging/Logging.h>

#include "NLohmannJSONConverterVisitor.h"

namespace armarx::aron::data::converter
{
    nlohmann::json
    AronNlohmannJSONConverter::ConvertToNlohmannJSON(const data::VariantPtr& aron)
    {
        nlohmann::json j;
        ConvertToNlohmannJSON(aron, j);
        return j;
    }

    void
    AronNlohmannJSONConverter::ConvertToNlohmannJSON(const aron::data::VariantPtr& aron,
                                                     nlohmann::json& j)
    {
        j = aron::data::readAndWrite<ToNLohmannJSONConverterVisitor>(aron);
    }

    data::DictPtr
    AronNlohmannJSONConverter::ConvertFromNlohmannJSONObject(const nlohmann::json& j,
                                                             const armarx::aron::Path& p)
    {
        // TODO Switch case over json type and add other methods, e.g. for float, array, ...
        // TODO add check if json is object_t
        data::VariantPtr aron = std::make_shared<aron::data::Dict>(p);
        ConvertFromNlohmannJSON(aron, j);
        return data::Dict::DynamicCastAndCheck(aron);
    }

    void
    AronNlohmannJSONConverter::ConvertFromNlohmannJSON(
        aron::data::VariantPtr& a,
        const nlohmann::json& e,
        const aron::type::VariantPtr& expectedStructure)
    {
        a = aron::data::readAndWrite<FromNLohmannJSONConverterVisitor>(e);
        ARMARX_CHECK_EXPRESSION(a->fullfillsType(expectedStructure));
    }

} // namespace armarx::aron::data::converter

namespace armarx::aron::type::converter
{

    nlohmann::json
    AronNlohmannJSONConverter::ConvertToNlohmannJSON(const type::VariantPtr& aron)
    {
        nlohmann::json j;
        ConvertToNlohmannJSON(aron, j);
        return j;
    }

    void
    AronNlohmannJSONConverter::ConvertToNlohmannJSON(const aron::type::VariantPtr& aron,
                                                     nlohmann::json& j)
    {
        j = aron::type::readAndWrite<ToNLohmannJSONConverterVisitor>(aron);
    }

    type::ObjectPtr
    AronNlohmannJSONConverter::ConvertFromNlohmannJSONTypeObject(const nlohmann::json& j,
                                                                 const armarx::aron::Path& p)
    {
        type::ObjectPtr foo =
            std::make_shared<aron::type::Object>("foo"); // foo is just a placeholder
        type::VariantPtr aron = std::make_shared<aron::type::Object>(*foo, p);
        ConvertFromNlohmannJSON(aron, j);
        return type::Object::DynamicCastAndCheck(aron);
    }

    void
    AronNlohmannJSONConverter::ConvertFromNlohmannJSON(aron::type::VariantPtr& a,
                                                       const nlohmann::json& e)
    {
        a = aron::type::readAndWrite<FromNLohmannJSONConverterVisitor>(e);
    }
} // namespace armarx::aron::type::converter
